ko.collaboration.friends = {};

(function() {
    this.update = function() {
        this.updateFriendsList();
        this.updateFriendRequests();
    };
    
    this.log = ko.logging.getLogger("ko.collaboration.friends");
    
    /**
     * List of all available friends. Gets initialized by updateFriendsList()
     * once friends are fetched from the server for the first time.
     */
    this.allFriends = null;
    
    /**
     * Prompts the user to supply a username and makes an API call to create a
     * friend request for that user.
     */
    this.sendFriendRequest = function sendFriendRequest() {
        var username = ko.dialogs.prompt("Please enter the email address of the contact you would like to add", "Email address",
                                         null, "Collaboration");
        if (username) {
            var this_ = this;
            let errorCallback = function(req) {
                if (req.status == 404) {
                    ko.dialogs.alert("User not found",
                                     "The email address you supplied does not belong to a known Komodo Collaboration user.");
                } else {
                    this_._displayError();
                }
            };
            ko.collaboration.api.addFriendAsync(username,
                                             ko.collaboration.utils.bind(this.update, this),
                                             errorCallback);
        }
    };
    
    /**
     * Fetches usernames of all friends and updates the friends list.
     */
    this.updateFriendsList = function updateFriendsList() {
        var this_ = this;
        ko.collaboration.api.getFriendsAsync(function(response) {
            if (!response.hasOwnProperty('friends')) {
                this_.log.error("Invalid response data from server: " +
                          JSON.stringify(response));
                this_._displayError();
            }
            this_._clearFriendsLists();
            this_.allFriends = {};
            for each (let [userId, userName] in Iterator(response.friends)) {
                this_._addFriend(userId, userName);
                this_.allFriends[userId] = userName;
            }
        }, this_._displayError);
    };
    
    this._clearFriendsLists = function _clearFriendsLists() {
        let list = document.getElementById('friends-list');
        if (list) {
            while (list.itemCount > 0) {
                list.removeItemAt(0);
            }
        }
    };
    
    this.friendRequests = {};
    
    this.updateFriendRequests = function updateFriendRequests() {
        // Get a list of friend requests
        // compare with stored list of friend requests:
        // For each new one, create a Notification, store, append
        // For each existing one, take the existing Notification, replace(n, n);
        
        let successCallback = ko.collaboration.utils.bind(function(response) {
            if (!response.hasOwnProperty('requests')) {
                this.log.error("Invalid response data from server: " +
                          JSON.stringify(response));
                this._displayError();
            }
            for each (let [userId, userName] in Iterator(response.requests)) {
                if (userId in this.friendRequests) {
                    this.reAddFriendRequestNotification(userId, userName);
                } else {
                    this.addFriendRequestNotification(userId, userName);
                }
            }
        }, this);
        
        ko.collaboration.api.getFriendRequestsAsync(successCallback,
                                                 this._displayError);
    };
    
    var _createReplyButton = function(username, confirm, label) {
        return new ko.notifications.NotificationButton(label, null, function() {
            var successCallback = function() {
                ko.collaboration.friends.updateFriendsList();
            }
            var errorCallback = function() {
                ko.collaboration.friends._displayError();
            };
            ko.collaboration.api.updateFriendRequestAsync(username, confirm,
                                                       successCallback,
                                                       errorCallback)
        });
    };
    
    this.addFriendRequestNotification = function(userId, userName) {
        var buttons = [_createReplyButton(userId, true, "Confirm"),
                       _createReplyButton(userId, false, "Deny")];
        var notification = new ko.notifications.Notification(
            "New friend request", userName + " wants add you as a " +
            "Collaboration contact.", null,
            null, buttons);
        ko.notifications.add(notification);
        // Notifications are never removed because we don't listen to the
        // notification:removed event.
        this.friendRequests[userId] = notification;
    };
    
    this.reAddFriendRequestNotification = function(userId, username) {
        if (!(userId in this.friendRequests))
            return;
        let notification = this.friendRequests[userId];
        if (ko.notifications.notifications.indexOf(notification) == -1) {
            ko.notifications.add(notification);
        }
    };
    
    /**
     * Adds an entry to the friends list.
     */
    this._addFriend = function _addFriend(userId, userName) {
        var friendsList = document.getElementById('friends-list');
        var item = document.createElement('richlistitem');
        item.setAttribute('class', 'friends-list-item');
        // Insert the node into the DOM before using the xbl magic.
        friendsList.appendChild(item);
        item.userId = userId;
        item.userName = userName;
    };
    
    /**
     * Adds an entry to the friend requests list.
     */
    this._addIncomingRequest = function _addIncomingRequest(username) {
        var incomingRequests = document.getElementById('incoming-requests-list');
        var item = document.createElement('richlistitem');
        item.setAttribute('class', 'incoming-request-list-item');
        // Insert the node into the DOM before using the xbl magic.
        incomingRequests.appendChild(item);
        item.username = username;
    };
    
    this._displayError = function _displayError() {
        this._clearFriendsLists();
        var item = document.createElement('richlistitem');
        item.value = 'An error occurred while trying to fetch your Contact ' +
                     'list. Please try again.';
        var friendsList = document.getElementById('friends-list');
        friendsList.appendChild(item);
    };
}).apply(ko.collaboration.friends);